require 'rspec'

require 'para_dice/bag'
require 'para_dice/numbered_die'
require 'para_dice/results/number'
describe ParaDice::Bag do
  let(:d4) { ParaDice::NumberedDie.get(4) }
  let(:d8) { ParaDice::NumberedDie.get(8) }
  let(:d12) { ParaDice::NumberedDie.get(12) }
  let(:bag) do
    bag = described_class.new
    bag.add_die d4
    bag.add_die d12
    bag.add_die d8
    bag.default_readers << ParaDice::Results::Number.new
    bag
  end

  context 'with all defaults' do
    subject { described_class.new }
    it do
      is_expected.to have_attributes(name:            'Dice Bag',
                                     dice:            {},
                                     default_readers: [])
    end
  end

  describe '.add_die(die)' do
    let(:new_die) { ParaDice::NumberedDie.get(20) }
    subject { described_class.new }
    specify { expect(subject.dice).to be_empty }
    it 'adds die to hash' do
      subject.add_die(new_die)
      expect(subject.dice['d20']).to eq new_die
    end
  end

  describe 'get_dice(*dice_names)' do
    let(:names) { ['d8', 'd8', 'd12'] }
    subject { bag.get_dice(*names) }
    it { expect(subject[0]).to eq d8 }
    it { expect(subject[1]).to eq d8 }
    it { expect(subject[2]).to eq d12 }
  end

  describe 'get_cup(*dice_names)' do
    let(:names) { ['d8', 'd8', 'd12'] }
    subject { bag.get_cup(*names) }
    it { is_expected.to be_a ParaDice::Cup }
    it { expect(subject.roll).to be_a Fixnum }
    it { expect(subject.roll).to be_between 3, 28 }
  end
end
