require 'rspec'

require 'para_dice/cup'
require 'para_dice/results/cancel_opposing'
require 'para_dice/results/hash'

describe ParaDice::Cup do

  describe 'initialize' do
    context 'with no arguments' do
      subject { described_class.new }

      it { is_expected.to have_attributes(dice: [], readers: [], rng: Random) }
    end

    context 'with rng seed' do
      subject { described_class.new(rng: Random.new(111)) }
      it { is_expected.to have_attributes(dice: [], readers: [], rng: Random) }

      it { expect(subject.rng.seed).to eq 111 }
    end
  end

  describe '.roll(roll_rng)' do
    let(:fbb) { ParaDice::StringDie.new(name: 'FBB', faces: [:foo, :bar, :baz]) }

    it 'call roll on all dice...' do
      cup = described_class.new(dice: [fbb, fbb, fbb])
      rng = double('rng')
      allow(rng).to receive(:rand).and_return(2, 0, 2)
      expect(rng).to receive(:rand).exactly(3).times
      cup.roll(rng)
    end

    it '...return array' do
      cup = described_class.new(dice: [fbb, fbb, fbb])
      rng = double('rng')
      allow(rng).to receive(:rand).and_return(2, 0, 2)
      expect(cup.roll(rng)).to eq ['baz', 'foo', 'baz']
    end
  end

  describe '.roll' do
    let(:fbb) { ParaDice::StringDie.new(name: 'FBB', faces: [:foo, :bar, :baz]) }
    let(:rng) do
      rng = double('rng')
      allow(rng).to receive(:rand).and_return(2, 0, 2, 1, 2)
      rng
    end
    let(:dice_list) { [fbb, fbb, fbb, fbb, fbb] }
    let(:cup) { described_class.new(dice: dice_list) }
    context 'with no readers' do
      subject { cup.roll(rng) }
      it { is_expected.to eq ['baz', 'foo', 'baz', 'bar', 'baz'] }
    end
    context 'with a hash_reader' do
      subject { cup.roll(rng) }
      let(:readers) { [ParaDice::Results::Hash] }
      let(:cup) { described_class.new(dice: dice_list, readers: readers) }
      it { is_expected.to eq('baz' => 3, 'foo' => 1, 'bar' => 1) }
    end
    context 'with a cancel_opposing reader' do
      subject { cup.roll(rng) }
      let(:cancel_out) { [['foo', 'baz']] }
      let(:readers) do
        [ParaDice::Results::CancelOpposing.new(cancel_out),
         ParaDice::Results::Hash]
      end
      let(:cup) { described_class.new(dice: dice_list, readers: readers) }
      it { is_expected.to eq('baz' => 2, 'bar' => 1) }
    end
  end
end
