require 'rspec'
require 'para_dice/die'
require 'para_dice/faces/ranged'

describe ParaDice::Faces::Ranged do

  before(:all) do
    class RangedFaceDie
      include ParaDice::Die
      include ParaDice::Faces::Ranged
    end
  end

  describe 'initialize' do
    let(:name) { 'd10-19' }
    let(:range) { 10..19 }
    subject { RangedFaceDie.new(name: name, range: range) }
    it { is_expected.to be_a ParaDice::Die }
    it do
      is_expected.to have_attributes(faces:      [10, 11, 12, 13, 14, 15, 16, 17, 18, 19],
                                        name:       name,
                                        range:      range,
                                        face_count: 10)
    end

    it 'will call rand with size on rng' do
      rng = double('random')
      expect(rng).to receive(:rand).with(10).and_return(2)
      subject.random_face(rng)
    end

    it 'will return the right value for a give rnd.rand result' do
      rng = double('random')
      allow(rng).to receive(:rand).with(10).and_return 4
      expect(subject.random_face(rng)).to eq 14
    end

    it 'will raise an error if no range is given to initializer' do
      expect { RangedFaceDie.new(name: 'oops').range }.to raise_error ArgumentError, /:range/
    end
  end
end
