require 'rspec'

require 'para_dice/faces/arrayed'

describe ParaDice::Faces::Arrayed do

  before(:all) do
    class StringFaceDie
      include ParaDice::Die
      include ParaDice::Faces::Arrayed
    end
  end

  describe 'initialize' do
    let(:name) { 'FBB' }
    let(:faces) { [:foo, :bar, :baz] }
    subject { StringFaceDie.new(name: name, faces: faces) }
    it { is_expected.to be_a ParaDice::Die }
    it do
      is_expected.to have_attributes(faces:      ['foo', 'bar', 'baz'],
                                     name:       name,
                                     face_count: 3)
    end

    it 'will call rand with size on rng' do
      rng = double('random')
      expect(rng).to receive(:rand).with(3).and_return(2)
      subject.random_face(rng)
    end

    it 'will return the right value for a give rnd.rand result' do
      rng = double('random')
      allow(rng).to receive(:rand).with(3).and_return 1
      expect(subject.random_face(rng)).to eq 'bar'
    end

    it 'will raise an error if no range is given to initializer' do
      expect { StringFaceDie.new(name: 'oops') }.to raise_error ArgumentError, /:faces/
    end
  end
end
