require 'rspec'
require 'para_dice/utility'

describe ParaDice::Utility do
  it { expect(ParaDice::Utility).to respond_to :module_from }

  describe '.module_from' do
    shared_examples_for 'a correct module' do |string, module_obj|
      let(:module_string) { string }

      it { is_expected.to be module_obj }
    end

    context 'without namespace' do
      subject { ParaDice::Utility.module_from(module_string) }

      context 'with top level module as string' do
        let(:module_string) { 'ParaDice' }
        it { is_expected.to be ParaDice }
      end

      context 'with deep module as string' do
        it_should_behave_like 'a correct module', 'ParaDice::Utility', ParaDice::Utility
        it_should_behave_like 'a correct module', 'para_dice__utility', ParaDice::Utility
        it_should_behave_like 'a correct module', 'ParaDice/Utility', ParaDice::Utility
      end
    end
    context 'with namespace' do
      let(:namespace) { ParaDice }

      subject { ParaDice::Utility.module_from(module_string, namespace) }

      context 'with top level module as string' do
        let(:module_string) { 'ParaDice' }
        it { is_expected.to be ParaDice }
      end

      context 'with deep module as string' do
        it_should_behave_like 'a correct module', 'ParaDice::Utility', ParaDice::Utility
        it_should_behave_like 'a correct module', 'para_dice__utility', ParaDice::Utility
        it_should_behave_like 'a correct module', 'ParaDice/Utility', ParaDice::Utility
        it_should_behave_like 'a correct module', 'Utility', ParaDice::Utility
        it_should_behave_like 'a correct module', 'utility', ParaDice::Utility
      end
    end

    context 'when theres no module with namd' do
      shared_examples_for 'an incorrect module' do |string, namespace_obj = nil|
        let(:module_string) { string }
        let(:namespace) { namespace_obj }
        subject { ParaDice::Utility.module_from(module_string, namespace) }

        it { is_expected.to be_falsey }
      end

      it_should_behave_like 'an incorrect module', 'BadName'
      it_should_behave_like 'an incorrect module', 'BadName', ParaDice
      it_should_behave_like 'an incorrect module', 'ParaDice__BadName'
      it_should_behave_like 'an incorrect module', 'bad_name', ParaDice
      it_should_behave_like 'an incorrect module', 'bad_name'

    end

    context 'when a class of module is passed' do
      it { expect(ParaDice::Utility.module_from(String)).to eq String }
      it { expect(ParaDice::Utility.module_from(ParaDice::Utility)).to eq ParaDice::Utility }
    end
  end

  describe '.to_summary' do
    let(:arr) { [3, 4, 3, 3, 4, 2, 1] }
    subject { ParaDice::Utility.to_summary(arr) }
    it { is_expected.to eq({ 3 => 3, 4 => 2, 2 => 1, 1 => 1 }) }
  end

  describe '.from_summary' do
    let(:summary) { { 3 => 3, 4 => 2, 2 => 1, 1 => 1 } }

    subject { ParaDice::Utility.from_summary(summary) }
    it { is_expected.to eq [3, 3, 3, 4, 4, 2, 1] }
  end
end
